package com.persagy.common.utils;

import java.util.List;

import com.alibaba.csp.sentinel.util.StringUtil;
import com.alibaba.fastjson.JSONObject;
import com.persagy.common.constant.DMPConstant;
import com.persagy.common.enums.ResponseCode;
import com.persagy.common.exception.BusinessException;

/**
 * @version
 * @description 对数据中台的响应体数据进行格式转变
 * @company persagy
 * @author zhangqiankun
 * @since 2020年10月19日:	上午11:03:24
 */
public class DMPResponseUtil {

    private static final String ERROR_MSG = "DMP服务响应异常";

    /**
     * 获取成功的响应数据
     * @param dmpResult
     * @return
     */
    public static ResponseResult convertResponseResult(String dmpResult) {
        if (StringUtil.isBlank(dmpResult) || dmpResult.indexOf(DMPConstant.DMP_RESULT_KEY) < 0) {
            return ResponseResultUtil.errorResult(ResponseCode.C0110.getCode(), ERROR_MSG);
        }
        JSONObject result = JSONObject.parseObject(dmpResult);
        if (DMPConstant.DMP_RESULT_VAL.equals(result.getString(DMPConstant.DMP_RESULT_KEY))) {
            return ResponseResultUtil.successResult(result.get(DMPConstant.DMP_DATA_KEY), result.getLong(DMPConstant.DMP_COUNT_KEY));
        } else if (StringUtil.isNotBlank(result.getString(DMPConstant.DMP_FAIL_MSG))) {
            return ResponseResultUtil.errorResult(ResponseCode.C0110.getCode(), result.getString(DMPConstant.DMP_FAIL_MSG));
        }

        return ResponseResultUtil.errorResult(ResponseCode.C0110.getCode(), ResponseCode.C0110.getDesc());
    }

    /**
     * 判断是否响应成功
     * @param dmpResult
     * @return
     */
    public static boolean isSuccess(String dmpResult) {
        if (StringUtil.isBlank(dmpResult) || dmpResult.indexOf(DMPConstant.DMP_RESULT_KEY) < 0) {
            return false;
        }
        JSONObject result = JSONObject.parseObject(dmpResult);
        if (DMPConstant.DMP_RESULT_VAL.equals(result.getString(DMPConstant.DMP_RESULT_KEY))) {
            return true;
        }
        return false;
    }

    /**
     * 判断中台正常响应的情况下，数据中的count值是否存在且大于0，非正常响应，抛出异常
     * @param dmpResult
     * @return 存在且大于0，返回true;否则返回false
     */
    public static boolean checkResultCount(String dmpResult) {
        ResponseResult responseResult = convertResponseResult(dmpResult);
        if(ResponseCode.A00000.getCode().equals(responseResult.getRespCode())){
            Long count = responseResult.getCount();
            if(null != count && count > 0){
                return true;
            }

            return false;
        }
        throw new BusinessException("数据中台处理失败：" + responseResult.getRespMsg());
    }

    /**
     * 正常响应转为对应的JSONObject,否则返回null
     * @param dmpResult
     * @return
     */
    public static JSONObject getJSONObject(String dmpResult) {
        if (StringUtil.isBlank(dmpResult) || dmpResult.indexOf(DMPConstant.DMP_RESULT_KEY) < 0) {
            return null;
        }

        JSONObject result = JSONObject.parseObject(dmpResult);
        if (DMPConstant.DMP_RESULT_VAL.equals(result.getString(DMPConstant.DMP_RESULT_KEY))) {
            // 响应成功，转变响应结构
            return result;
        }
        return null;
    }

    /**
     * @param dmpResult
     * @return 正常count 否则 null
     */
    public static Long getDMPResultCount(String dmpResult) {
        JSONObject jsonObject = getJSONObject(dmpResult);
        return jsonObject.getLong(DMPConstant.DMP_COUNT_KEY);
    }

    /***
     * @Description: 转换结果
     * @param content : 数组形式的
     * @param clazz : 类的类型
     * @return : java.util.List<T>
     * @author: lijie
     * @Date:2020/10/19 14:59
     * Update By lijie 2020/10/19 14:59
     */
    public static <T> List<T> transferObjectToList(Object content, Class<T> clazz){
        return JSONObject.parseArray(JSONObject.toJSONString(content), clazz);
    }

    /***
     * @Description: 转换结果
     * @param content : 对象形式的
     * @param clazz : 类的类型
     * @return : java.util.List<T>
     * @author: lijie
     * @Date:2020/10/19 14:59
     * Update By lijie 2020/10/19 14:59
     */
    public static <T> T transferObjectToObject(Object content, Class<T> clazz){
        return JSONObject.parseObject(JSONObject.toJSONString(content), clazz);
    }

}
